@@ -61,6 +61,8 @@
 #include <QToolBar>
 #include <QVBoxLayout>
 
+#include <utility>
+
 MainWindow::MainWindow(QWidget *parent)
     : KXmlGuiWindow(parent)
     , m_protocolInput(nullptr)
@@ -153,6 +155,12 @@ void MainWindow::setupActions()
     viewOnlyAction->setIcon(QIcon::fromTheme(QStringLiteral("document-preview")));
     connect(viewOnlyAction, SIGNAL(triggered(bool)), SLOT(viewOnly(bool)));
 
+    QAction *clipboardSharingAction = actionCollection()->addAction(QStringLiteral("clipboard_sharing"));
+    clipboardSharingAction->setCheckable(true);
+    clipboardSharingAction->setText(i18n("Clipboard Sharing"));
+    clipboardSharingAction->setIcon(QIcon::fromTheme(QStringLiteral("edit-copy")));
+    connect(clipboardSharingAction, SIGNAL(triggered(bool)), SLOT(clipboardSharing(bool)));
+
     QAction *disconnectAction = actionCollection()->addAction(QStringLiteral("disconnect"));
     disconnectAction->setText(i18n("Disconnect"));
     disconnectAction->setIcon(QIcon::fromTheme(QStringLiteral("network-disconnect")));
@@ -249,7 +257,7 @@ void MainWindow::newConnection(const QUrl &newUrl, bool switchFullscreenWhenConn
 
     if (url.isLocalFile()) {
         QUrl loadedUrl;
-        for (RemoteViewFactory *factory : qAsConst(m_remoteViewFactories)) {
+        for (RemoteViewFactory *factory : std::as_const(m_remoteViewFactories)) {
             loadedUrl = factory->loadUrlFromFile(url);
             if (loadedUrl.isValid()) {
                 qCDebug(KRDC) << "Loaded file (" << url.path() << ") resulted in url (" << loadedUrl << ") using " << factory->metaObject()->className();
@@ -277,7 +285,7 @@ void MainWindow::newConnection(const QUrl &newUrl, bool switchFullscreenWhenConn
     RemoteView *view = nullptr;
     KConfigGroup configGroup = Settings::self()->config()->group(QStringLiteral("hostpreferences")).group(url.toDisplayString(QUrl::StripTrailingSlash));
 
-    for (RemoteViewFactory *factory : qAsConst(m_remoteViewFactories)) {
+    for (RemoteViewFactory *factory : std::as_const(m_remoteViewFactories)) {
         if (factory->supportsUrl(url)) {
             view = factory->createView(this, url, configGroup);
             qCDebug(KRDC) << "Found plugin to handle url (" << url.url() << "): " << view->metaObject()->className();
@@ -490,7 +498,7 @@ void MainWindow::switchFullscreen()
 
         KToggleFullScreenAction::setFullScreen(this, false);
 
-        for (RemoteView *view : qAsConst(m_remoteViewMap)) {
+        for (RemoteView *view : std::as_const(m_remoteViewMap)) {
             view->switchFullscreen(false);
             view->enableScaling(view->hostPreferences()->windowedScale());
         }
@@ -527,7 +535,7 @@ void MainWindow::switchFullscreen()
         MinimizePixel *minimizePixel = new MinimizePixel(this);
         connect(minimizePixel, SIGNAL(rightClicked()), this, SLOT(showMinimized()));
 
-        for (RemoteView *currentView : qAsConst(m_remoteViewMap)) {
+        for (RemoteView *currentView : std::as_const(m_remoteViewMap)) {
             currentView->enableScaling(currentView->hostPreferences()->fullscreenScale());
             currentView->switchFullscreen(true);
         }
@@ -652,7 +660,7 @@ void MainWindow::showSettingsDialog(const QString &url)
 {
     HostPreferences *prefs = nullptr;
 
-    for (RemoteViewFactory *factory : qAsConst(m_remoteViewFactories)) {
+    for (RemoteViewFactory *factory : std::as_const(m_remoteViewFactories)) {
         if (factory->supportsUrl(QUrl(url))) {
             prefs = factory->createHostPreferences(Settings::self()->config()->group(QStringLiteral("hostpreferences")).group(url), this);
             if (prefs) {
@@ -768,6 +776,15 @@ void MainWindow::viewOnly(bool viewOnly)
     saveHostPrefs(view);
 }
 
+void MainWindow::clipboardSharing(bool clipboardSharing)
+{
+    qCDebug(KRDC) << clipboardSharing;
+
+    RemoteView *view = currentRemoteView();
+    view->hostPreferences()->setClipboardSharing(clipboardSharing);
+    saveHostPrefs(view);
+}
+
 void MainWindow::grabAllKeys(bool grabAllKeys)
 {
     qCDebug(KRDC);
@@ -846,6 +863,7 @@ void MainWindow::showRemoteViewToolbar()
 
         buttonBox->addAction(actionCollection()->action(QStringLiteral("take_screenshot")));
         buttonBox->addAction(actionCollection()->action(QStringLiteral("view_only")));
+        buttonBox->addAction(actionCollection()->action(QStringLiteral("clipboard_sharing")));
         buttonBox->addAction(actionCollection()->action(QStringLiteral("show_local_cursor")));
         buttonBox->addAction(actionCollection()->action(QStringLiteral("grab_all_keys")));
         buttonBox->addAction(actionCollection()->action(QStringLiteral("scale")));
@@ -880,6 +898,11 @@ void MainWindow::updateActionStatus()
 
     setActionStatus(actionCollection()->action(QStringLiteral("view_only")), enabled, view ? view->supportsViewOnly() : false, view ? view->viewOnly() : false);
 
+    setActionStatus(actionCollection()->action(QStringLiteral("clipboard_sharing")),
+                    enabled,
+                    view ? view->supportsClipboardSharing() : false,
+                    view ? view->hostPreferences()->clipboardSharing() : false);
+
     setActionStatus(actionCollection()->action(QStringLiteral("show_local_cursor")),
                     enabled,
                     view ? view->supportsLocalCursor() : false,
@@ -948,7 +971,7 @@ void MainWindow::updateConfiguration()
     }
 
     // Send update configuration message to all views
-    for (RemoteView *view : qAsConst(m_remoteViewMap)) {
+    for (RemoteView *view : std::as_const(m_remoteViewMap)) {
         view->updateConfiguration();
     }
 }
@@ -966,7 +989,7 @@ void MainWindow::quit(bool systemEvent)
             == KMessageBox::Continue) {
         if (Settings::rememberSessions()) { // remember open remote views for next startup
             QStringList list;
-            for (RemoteView *view : qAsConst(m_remoteViewMap)) {
+            for (RemoteView *view : std::as_const(m_remoteViewMap)) {
                 qCDebug(KRDC) << view->url();
                 list.append(view->url().toDisplayString(QUrl::StripTrailingSlash));
             }
@@ -1031,7 +1054,7 @@ void MainWindow::saveProperties(KConfigGroup &group)
 
 void MainWindow::saveHostPrefs()
 {
-    for (RemoteView *view : qAsConst(m_remoteViewMap)) {
+    for (RemoteView *view : std::as_const(m_remoteViewMap)) {
         saveHostPrefs(view);
     }
 }
@@ -1094,7 +1117,7 @@ QWidget *MainWindow::newConnectionWidget()
         m_addressInput->setPlaceholderText(i18n("Type here to connect to an address and filter the list."));
         connect(m_addressInput, SIGNAL(textChanged(QString)), remoteDesktopsModelProxy, SLOT(setFilterFixedString(QString)));
 
-        for (RemoteViewFactory *factory : qAsConst(m_remoteViewFactories)) {
+        for (RemoteViewFactory *factory : std::as_const(m_remoteViewFactories)) {
             m_protocolInput->addItem(factory->scheme());
         }
         m_protocolInput->setCurrentText(Settings::defaultProtocol());
